@@ -35,9 +35,7 @@
 
 #include <filesystem>
 
-#include <zeep/nvp.hpp>
-
-#include "cif++/Structure.hpp"
+#include <cif++.hpp>
 
 #if __has_include(<experimental/optional>)
 #include <experimental/optional>
@@ -60,18 +58,6 @@ struct ResidueSpec
 	std::string label_comp_id;
 	int label_seq_id;
 
-	template <typename Archive>
-	void serialize(Archive &ar, unsigned long version)
-	{
-		ar	& ZEEP_SERIALIZATION_NVP(auth_asym_id)
-		  	& ZEEP_SERIALIZATION_NVP(auth_comp_id)
-		   	& ZEEP_SERIALIZATION_NVP(auth_seq_id)
-		   	& ZEEP_SERIALIZATION_NVP(pdbx_PDB_ins_code)
-		   	& ZEEP_SERIALIZATION_NVP(label_asym_id)
-		   	& ZEEP_SERIALIZATION_NVP(label_comp_id)
-		   	& ZEEP_SERIALIZATION_NVP(label_seq_id);
-	}
-
 	ResidueSpec() {}
 	ResidueSpec(const std::string &auth_asym_id,
 		const std::string &auth_comp_id,
@@ -97,28 +83,28 @@ struct ResidueSpec
 	ResidueSpec(const ResidueSpec &rhs) = default;
 	ResidueSpec &operator=(const ResidueSpec &rhs) = default;
 
-	ResidueSpec(const mmcif::Residue &res)
-		: auth_asym_id(res.authAsymID())
-		, auth_comp_id(res.compoundID())
-		, auth_seq_id(res.authSeqID())
-		, label_asym_id(res.asymID())
-		, label_comp_id(res.compoundID())
-		, label_seq_id(res.seqID())
+	ResidueSpec(const cif::mm::residue &res)
+		: auth_asym_id(res.get_auth_asym_id())
+		, auth_comp_id(res.get_compound_id())
+		, auth_seq_id(res.get_auth_seq_id())
+		, label_asym_id(res.get_asym_id())
+		, label_comp_id(res.get_compound_id())
+		, label_seq_id(res.get_seq_id())
 	{
-		char ins_code = res.authInsCode().c_str()[0];
+		char ins_code = res.get_pdb_ins_code().c_str()[0];
 		if (ins_code != 0 and ins_code != ' ')
 			pdbx_PDB_ins_code = ins_code;
 	}
 
-	ResidueSpec(const mmcif::Atom &atom)
-		: auth_asym_id(atom.authAsymID())
-		, auth_comp_id(atom.authCompID())
-		, auth_seq_id(atom.authSeqID())
-		, label_asym_id(atom.labelAsymID())
-		, label_comp_id(atom.labelCompID())
-		, label_seq_id(atom.labelSeqID())
+	ResidueSpec(const cif::mm::atom &atom)
+		: auth_asym_id(atom.get_auth_asym_id())
+		, auth_comp_id(atom.get_label_comp_id())
+		, auth_seq_id(atom.get_auth_seq_id())
+		, label_asym_id(atom.get_label_asym_id())
+		, label_comp_id(atom.get_label_comp_id())
+		, label_seq_id(atom.get_label_seq_id())
 	{
-		char ins_code = atom.pdbxAuthInsCode().c_str()[0];
+		char ins_code = atom.get_pdb_ins_code().c_str()[0];
 		if (ins_code != 0 and ins_code != ' ')
 			pdbx_PDB_ins_code = ins_code;
 	}
@@ -140,7 +126,6 @@ using SkipList = std::vector<ResidueSpec>;
 enum class SkipListFormat
 {
 	OLD,
-	JSON,
 	CIF
 };
 
