@@ -26,7 +26,7 @@
 
 #pragma once
 
-#include "cif++/BondMap.hpp"
+#include <pdb-redo/BondMap.hpp>
 #include "pdb-redo/DistanceMap.hpp"
 #include "pdb-redo/MapMaker.hpp"
 
@@ -53,6 +53,31 @@ std::ostream &operator<<(std::ostream &os, const ResidueStatistics &st);
 
 // --------------------------------------------------------------------
 
+template <typename F, typename FTYPE = float>
+void iterateGrid(const clipper::Coord_orth &p, float r, const clipper::Xmap<FTYPE> &m, F &&func)
+{
+	using namespace clipper;
+
+	Coord_frac fp = p.coord_frac(m.cell());
+
+	Coord_frac o = Coord_orth(r, r, r).coord_frac(m.cell());
+	o[0] = std::abs(o[0]);
+	o[1] = std::abs(o[1]);
+	o[2] = std::abs(o[2]);
+
+	Coord_frac fMin = fp - o, fMax = fp + o;
+	Coord_map mMin = fMin.coord_map(m.grid_sampling()), mMax = fMax.coord_map(m.grid_sampling());
+	Coord_grid gMin = mMin.floor(), gMax = mMax.ceil();
+
+	auto i0 = Xmap_base::Map_reference_coord(m, gMin);
+	for (auto iu = i0; iu.coord().u() <= gMax[0]; iu.next_u())
+		for (auto iv = iu; iv.coord().v() <= gMax[1]; iv.next_v())
+			for (auto iw = iv; iw.coord().w() <= gMax[2]; iw.next_w())
+				func(iw);
+}
+
+// --------------------------------------------------------------------
+
 class StatsCollector
 {
   public:
@@ -60,24 +85,24 @@ class StatsCollector
 	StatsCollector &operator=(const StatsCollector &) = delete;
 
 	StatsCollector(const MapMaker<float> &mm,
-		mmcif::Structure &structure, bool electronScattering);
+		cif::mm::structure &structure, bool electronScattering);
 
 	virtual std::vector<ResidueStatistics> collect() const;
 
 	virtual std::vector<ResidueStatistics> collect(const std::string &asymID,
 		int resFirst, int resLast, bool authNameSpace = true) const;
 
-	virtual ResidueStatistics collect(std::initializer_list<const mmcif::Residue *> residues) const;
+	virtual ResidueStatistics collect(std::initializer_list<const cif::mm::residue *> residues) const;
 
-	virtual ResidueStatistics collect(std::initializer_list<mmcif::Atom> atoms) const;
+	virtual ResidueStatistics collect(std::initializer_list<cif::mm::atom> atoms) const;
 
-	virtual ResidueStatistics collect(const std::vector<mmcif::Atom> &atoms) const;
+	virtual ResidueStatistics collect(const std::vector<cif::mm::atom> &atoms) const;
 
   protected:
+	using residue_list = std::vector<std::tuple<std::string, int, std::string>>;
+
 	// asym-seqid-compid
-	std::vector<ResidueStatistics> collect(
-		const std::vector<std::tuple<std::string, int, std::string, std::string>> &residues,
-		BoundingBox &bbox, bool addWaters) const;
+	std::vector<ResidueStatistics> collect(const residue_list &residues, BoundingBox &bbox, bool addWaters) const;
 
 	void initialize();
 
@@ -98,7 +123,7 @@ class StatsCollector
 
 	typedef std::map<clipper::Coord_grid, double, cmpGPt> GridPtDataMap;
 
-	mmcif::Structure &mStructure;
+	cif::mm::structure &mStructure;
 	const MapMaker<float> &mMapMaker;
 
 	clipper::Spacegroup mSpacegroup;
@@ -127,15 +152,15 @@ class EDIAStatsCollector : public StatsCollector
 {
   public:
 	EDIAStatsCollector(MapMaker<float> &mm,
-		mmcif::Structure &structure, bool electronScattering,
-		const mmcif::BondMap &bondMap);
+		cif::mm::structure &structure, bool electronScattering,
+		const BondMap &bondMap);
 
   protected:
 	virtual void calculate(std::vector<AtomData> &atomData) const;
 
 	DistanceMap mDistanceMap;
-	const mmcif::BondMap &mBondMap;
-	std::map<mmcif::AtomType, float> mRadii;
+	const BondMap &mBondMap;
+	std::map<cif::atom_type, float> mRadii;
 };
 
 } // namespace pdb_redo
