@@ -29,50 +29,18 @@
 #include <numeric>
 #include <shared_mutex>
 
-#include <boost/algorithm/string.hpp>
-
 #include <filesystem>
 #include <fstream>
 
-#include "cif++/Cif++.hpp"
-#include "cif++/CifUtils.hpp"
-#include "cif++/Compound.hpp"
-#include "cif++/Point.hpp"
-
 #include "pdb-redo/Compound.hpp"
 
-namespace ba = boost::algorithm;
 namespace fs = std::filesystem;
 
 namespace pdb_redo
 {
 
-using mmcif::kPI;
-using mmcif::AtomTypeTraits;
-
-// --------------------------------------------------------------------
-// Factory class for Compound and Link objects
-
-class CompoundFactory
-{
-  public:
-	static CompoundFactory &instance();
-
-	const Compound *get(std::string id);
-	const Compound *create(std::string id);
-
-	const Link *getLink(std::string id);
-	const Link *createLink(std::string id);
-
-	~CompoundFactory();
-
-	void pushDictionary(const std::filesystem::path &inDictFile);
-
-  private:
-	CompoundFactory();
-
-	class CompoundFactoryImpl *mImpl;
-};
+using cif::kPI;
+using cif::atom_type_traits;
 
 // --------------------------------------------------------------------
 // Compound helper classes
@@ -121,12 +89,12 @@ Compound::Compound(const std::string &file, const std::string &id,
 
 		for (auto row : compoundAtoms)
 		{
-			std::string id, symbol, energy;
+			std::string atom_id, symbol, energy;
 			float charge;
 
-			cif::tie(id, symbol, energy, charge) = row.get("atom_id", "type_symbol", "type_energy", "partial_charge");
+			cif::tie(atom_id, symbol, energy, charge) = row.get("atom_id", "type_symbol", "type_energy", "partial_charge");
 
-			mAtoms.push_back({id, mmcif::AtomTypeTraits(symbol).type(), energy, charge});
+			mAtoms.push_back({atom_id, atom_type_traits(symbol).type(), energy, charge});
 		}
 		sort(mAtoms.begin(), mAtoms.end(), CompoundAtomLess());
 
@@ -152,7 +120,7 @@ Compound::Compound(const std::string &file, const std::string &id,
 				b.type = delocalizedBond;
 			else
 			{
-				if (cif::VERBOSE)
+				if (cif::VERBOSE > 0)
 					std::cerr << "Unimplemented chem_comp_bond.type " << type << " in " << id << std::endl;
 				b.type = singleBond;
 			}
@@ -202,7 +170,7 @@ Compound::Compound(const std::string &file, const std::string &id,
 				cc.volumeSign = both;
 			else
 			{
-				if (cif::VERBOSE)
+				if (cif::VERBOSE > 0)
 					std::cerr << "Unimplemented chem_comp_chir.volume_sign " << volumeSign << " in " << id << std::endl;
 				continue;
 			}
@@ -243,7 +211,7 @@ std::string Compound::formula() const
 
 	for (auto r : mAtoms)
 	{
-		atoms[mmcif::AtomTypeTraits(r.typeSymbol).symbol()] += 1;
+		atoms[atom_type_traits(r.typeSymbol).symbol()] += 1;
 		chargeSum += r.partialCharge;
 	}
 
@@ -290,7 +258,7 @@ float Compound::formulaWeight() const
 	float result = 0;
 
 	for (auto r : mAtoms)
-		result += AtomTypeTraits(r.typeSymbol).weight();
+		result += atom_type_traits(r.typeSymbol).weight();
 
 	return result;
 }
@@ -352,7 +320,7 @@ bool Compound::isSugar() const
 	return cif::iequals(mGroup, "furanose") or cif::iequals(mGroup, "pyranose");
 }
 
-CompoundAtom Compound::getAtomByID(const std::string &atomID) const
+CompoundAtom Compound::get_atom_by_atom_id(const std::string &atomID) const
 {
 	CompoundAtom result = {};
 	for (auto &a : mAtoms)
@@ -522,10 +490,10 @@ float Compound::bondAngle(const std::string &atomId_1, const std::string &atomId
 
 // static float calcC(float a, float b, float alpha)
 //{
-//	float f = b * sin(alpha * kPI / 180);
-//	float d = sqrt(b * b - f * f);
+//	float f = b * std::sin(alpha * kPI / 180);
+//	float d = std::sqrt(b * b - f * f);
 //	float e = a - d;
-//	float c = sqrt(f * f + e * e);
+//	float c = std::sqrt(f * f + e * e);
 //
 //	return c;
 // }
@@ -553,11 +521,11 @@ float Compound::chiralVolume(const std::string &centreID) const
 		float beta = bondAngle(cv.atomID[1], cv.atomIDCentre, cv.atomID[2]);
 		float gamma = bondAngle(cv.atomID[2], cv.atomIDCentre, cv.atomID[0]);
 
-		float cosa = cos(alpha * kPI / 180);
-		float cosb = cos(beta * kPI / 180);
-		float cosc = cos(gamma * kPI / 180);
+		float cosa = static_cast<float>(std::cos(alpha * kPI / 180));
+		float cosb = static_cast<float>(std::cos(beta * kPI / 180));
+		float cosc = static_cast<float>(std::cos(gamma * kPI / 180));
 
-		result = (a * b * c * sqrt(1 + 2 * cosa * cosb * cosc - (cosa * cosa) - (cosb * cosb) - (cosc * cosc))) / 6;
+		result = (a * b * c * std::sqrt(1 + 2 * cosa * cosb * cosc - (cosa * cosa) - (cosb * cosb) - (cosc * cosc))) / 6;
 
 		if (cv.volumeSign == negativ)
 			result = -result;
@@ -570,9 +538,9 @@ float Compound::chiralVolume(const std::string &centreID) const
 
 // --------------------------------------------------------------------
 
-Link::Link(cif::Datablock &db)
+Link::Link(cif::datablock &db)
 {
-	mID = db.getName();
+	mID = db.name();
 
 	auto &linkBonds = db["chem_link_bond"];
 
@@ -598,7 +566,7 @@ Link::Link(cif::Datablock &db)
 			b.type = delocalizedBond;
 		else
 		{
-			if (cif::VERBOSE)
+			if (cif::VERBOSE > 0)
 				std::cerr << "Unimplemented chem_link_bond.type " << type << " in " << mID << std::endl;
 			b.type = singleBond;
 		}
@@ -660,7 +628,7 @@ Link::Link(cif::Datablock &db)
 			cc.volumeSign = both;
 		else
 		{
-			if (cif::VERBOSE)
+			if (cif::VERBOSE > 0)
 				std::cerr << "Unimplemented chem_link_chir.volume_sign " << volumeSign << " in " << mID << std::endl;
 			continue;
 		}
@@ -734,10 +702,50 @@ float Link::bondAngle(const LinkAtom &atom1, const LinkAtom &atom2, const LinkAt
 	return result;
 }
 
-float Link::chiralVolume(const std::string &centreID) const
+float Link::chiralVolume(const std::string &centreID, const std::string &compound_id_1, const std::string &compound_id_2) const
 {
 	float result = 0;
 
+	auto bondValue = [&](const LinkAtom &a, const LinkAtom &b) -> float
+	{
+		float result = atomBondValue(a, b);
+
+		if (result == 0)
+		{
+			if (a.compID != b.compID)
+				throw std::runtime_error("cannot calculate chiral volume since bond lengths are missing");
+			
+			auto cmp = Compound::create(a.compID == 1 ? compound_id_1 : compound_id_2);
+
+			if (cmp == nullptr)
+				throw std::runtime_error("cannot calculate chiral volume since compound is not known");
+
+			result = cmp->atomBondValue(a.atomID, b.atomID);
+		}
+
+		return result;
+	};
+
+	auto angle = [&](const LinkAtom &a, const LinkAtom &b, const LinkAtom &c) -> float
+	{
+		float result = bondAngle(a, b, c);
+
+		if (std::isnan(result))
+		{
+			if (a.compID != b.compID or a.compID != c.compID)
+				throw std::runtime_error("cannot calculate chiral volume since bond lengths are missing");
+			
+			auto cmp = Compound::create(a.compID == 1 ? compound_id_1 : compound_id_2);
+
+			if (cmp == nullptr)
+				throw std::runtime_error("cannot calculate chiral volume since compound is not known");
+
+			result = cmp->bondAngle(a.atomID, b.atomID, c.atomID);
+		}
+
+		return result;
+	};
+
 	for (auto &cv : mChiralCentres)
 	{
 		if (cv.id != centreID)
@@ -747,21 +755,21 @@ float Link::chiralVolume(const std::string &centreID) const
 
 		// the edges
 
-		float a = atomBondValue(cv.atomCentre, cv.atom[0]);
-		float b = atomBondValue(cv.atomCentre, cv.atom[1]);
-		float c = atomBondValue(cv.atomCentre, cv.atom[2]);
+		float a = bondValue(cv.atomCentre, cv.atom[0]);
+		float b = bondValue(cv.atomCentre, cv.atom[1]);
+		float c = bondValue(cv.atomCentre, cv.atom[2]);
 
 		// the angles for the top of the tetrahedron
 
-		float alpha = bondAngle(cv.atom[0], cv.atomCentre, cv.atom[1]);
-		float beta = bondAngle(cv.atom[1], cv.atomCentre, cv.atom[2]);
-		float gamma = bondAngle(cv.atom[2], cv.atomCentre, cv.atom[0]);
+		float alpha = angle(cv.atom[0], cv.atomCentre, cv.atom[1]);
+		float beta = angle(cv.atom[1], cv.atomCentre, cv.atom[2]);
+		float gamma = angle(cv.atom[2], cv.atomCentre, cv.atom[0]);
 
-		float cosa = cos(alpha * kPI / 180);
-		float cosb = cos(beta * kPI / 180);
-		float cosc = cos(gamma * kPI / 180);
+		float cosa = static_cast<float>(std::cos(alpha * kPI / 180));
+		float cosb = static_cast<float>(std::cos(beta * kPI / 180));
+		float cosc = static_cast<float>(std::cos(gamma * kPI / 180));
 
-		result = (a * b * c * sqrt(1 + 2 * cosa * cosb * cosc - (cosa * cosa) - (cosb * cosb) - (cosc * cosc))) / 6;
+		result = (a * b * c * std::sqrt(1 + 2 * cosa * cosb * cosc - (cosa * cosa) - (cosb * cosb) - (cosc * cosc))) / 6;
 
 		if (cv.volumeSign == negativ)
 			result = -result;
@@ -817,7 +825,8 @@ class CompoundFactoryImpl
   public:
 	CompoundFactoryImpl();
 
-	CompoundFactoryImpl(const std::string &file, CompoundFactoryImpl *next);
+	CompoundFactoryImpl(const std::filesystem::path &file, CompoundFactoryImpl *next);
+	CompoundFactoryImpl(std::istream &data, CompoundFactoryImpl *next);
 
 	~CompoundFactoryImpl()
 	{
@@ -842,15 +851,15 @@ class CompoundFactoryImpl
 	{
 		std::string result = resName;
 
-		auto &e = const_cast<cif::File &>(mFile)["comp_synonym_list"];
+		auto &e = const_cast<cif::file &>(mFile)["comp_synonym_list"];
 
-		for (auto &synonym : e["chem_comp_synonyms"])
+		for (auto synonym : e["chem_comp_synonyms"])
 		{
-			if (ba::iequals(synonym["comp_alternative_id"].as<std::string>(), resName) == false)
+			if (cif::iequals(synonym["comp_alternative_id"].as<std::string>(), resName) == false)
 				continue;
 
 			result = synonym["comp_id"].as<std::string>();
-			ba::trim(result);
+			cif::trim(result);
 			break;
 		}
 
@@ -881,7 +890,7 @@ class CompoundFactoryImpl
 	std::set<std::string> mKnownPeptides;
 	std::set<std::string> mKnownBases;
 	std::set<std::string> mMissing;
-	cif::File mFile;
+	cif::file mFile;
 	CompoundFactoryImpl *mNext = nullptr;
 };
 
@@ -896,16 +905,16 @@ CompoundFactoryImpl::CompoundFactoryImpl()
 		mKnownBases.insert(key);
 }
 
-CompoundFactoryImpl::CompoundFactoryImpl(const std::string &file, CompoundFactoryImpl *next)
-	: mPath(file)
+CompoundFactoryImpl::CompoundFactoryImpl(const std::filesystem::path &file, CompoundFactoryImpl *next)
+	: mPath(file.string())
 	, mFile(file)
 	, mNext(next)
 {
 	const std::regex peptideRx("(?:[lmp]-)?peptide", std::regex::icase);
 
-	auto &cat = mFile.firstDatablock()["chem_comp"];
+	auto &cat = mFile["comp_list"]["chem_comp"];
 
-	for (auto &chemComp : cat)
+	for (auto chemComp : cat)
 	{
 		std::string group, threeLetterCode;
 
@@ -913,7 +922,28 @@ CompoundFactoryImpl::CompoundFactoryImpl(const std::string &file, CompoundFactor
 
 		if (std::regex_match(group, peptideRx))
 			mKnownPeptides.insert(threeLetterCode);
-		else if (ba::iequals(group, "DNA") or ba::iequals(group, "RNA"))
+		else if (cif::iequals(group, "DNA") or cif::iequals(group, "RNA"))
+			mKnownBases.insert(threeLetterCode);
+	}
+}
+
+CompoundFactoryImpl::CompoundFactoryImpl(std::istream &data, CompoundFactoryImpl *next)
+	: mFile(data)
+	, mNext(next)
+{
+	const std::regex peptideRx("(?:[lmp]-)?peptide", std::regex::icase);
+
+	auto &cat = mFile["comp_list"]["chem_comp"];
+
+	for (auto chemComp : cat)
+	{
+		std::string group, threeLetterCode;
+
+		cif::tie(group, threeLetterCode) = chemComp.get("group", "three_letter_code");
+
+		if (std::regex_match(group, peptideRx))
+			mKnownPeptides.insert(threeLetterCode);
+		else if (cif::iequals(group, "DNA") or cif::iequals(group, "RNA"))
 			mKnownBases.insert(threeLetterCode);
 	}
 }
@@ -922,7 +952,7 @@ Compound *CompoundFactoryImpl::get(std::string id)
 {
 	std::shared_lock lock(mMutex);
 
-	ba::to_upper(id);
+	cif::to_upper(id);
 
 	Compound *result = nullptr;
 
@@ -943,16 +973,16 @@ Compound *CompoundFactoryImpl::get(std::string id)
 
 Compound *CompoundFactoryImpl::create(std::string id)
 {
-	ba::to_upper(id);
+	cif::to_upper(id);
 
 	Compound *result = get(id);
 	if (result == nullptr and mMissing.count(id) == 0 and not mFile.empty())
 	{
 		std::unique_lock lock(mMutex);
 
-		auto &cat = mFile.firstDatablock()["chem_comp"];
+		auto &cat = mFile["comp_list"]["chem_comp"];
 
-		auto rs = cat.find(cif::Key("three_letter_code") == id);
+		auto rs = cat.find(cif::key("three_letter_code") == id);
 
 		if (not rs.empty())
 		{
@@ -963,17 +993,17 @@ Compound *CompoundFactoryImpl::create(std::string id)
 			cif::tie(name, group, numberAtomsAll, numberAtomsNh) =
 				row.get("name", "group", "number_atoms_all", "number_atoms_nh");
 
-			ba::trim(name);
-			ba::trim(group);
+			cif::trim(name);
+			cif::trim(group);
 
-			if (mFile.get("comp_" + id) == nullptr)
+			if (not mFile.contains("comp_" + id))
 			{
 				auto clibd_mon = fs::path(getenv("CLIBD_MON"));
 
-				fs::path resFile = clibd_mon / ba::to_lower_copy(id.substr(0, 1)) / (id + ".cif");
+				fs::path resFile = clibd_mon / cif::to_lower_copy(id.substr(0, 1)) / (id + ".cif");
 
 				if (not fs::exists(resFile) and (id == "COM" or id == "CON" or "PRN")) // seriously...
-					resFile = clibd_mon / ba::to_lower_copy(id.substr(0, 1)) / (id + '_' + id + ".cif");
+					resFile = clibd_mon / cif::to_lower_copy(id.substr(0, 1)) / (id + '_' + id + ".cif");
 
 				if (not fs::exists(resFile))
 					mMissing.insert(id);
@@ -1001,7 +1031,7 @@ const Link *CompoundFactoryImpl::getLink(std::string id)
 {
 	std::shared_lock lock(mMutex);
 
-	ba::to_upper(id);
+	cif::to_upper(id);
 
 	const Link *result = nullptr;
 
@@ -1022,7 +1052,7 @@ const Link *CompoundFactoryImpl::getLink(std::string id)
 
 const Link *CompoundFactoryImpl::createLink(std::string id)
 {
-	ba::to_upper(id);
+	cif::to_upper(id);
 
 	const Link *result = getLink(id);
 
@@ -1030,10 +1060,9 @@ const Link *CompoundFactoryImpl::createLink(std::string id)
 	{
 		std::unique_lock lock(mMutex);
 
-		auto db = mFile.get("link_" + id);
-		if (db != nullptr)
+		if (mFile.contains("link_" + id))
 		{
-			result = new Link(*db);
+			result = new Link(mFile["link_" + id]);
 			mLinks.push_back(result);
 		}
 
@@ -1059,7 +1088,7 @@ CompoundFactory::CompoundFactory()
 
 	if (mImpl == nullptr)
 	{
-		if (cif::VERBOSE)
+		if (cif::VERBOSE > 0)
 			std::cerr << "Could not load the mon_lib_list.cif file from CCP4, please make sure you have installed CCP4 and sourced the environment." << std::endl;
 
 		mImpl = new CompoundFactoryImpl();
@@ -1084,7 +1113,7 @@ void CompoundFactory::pushDictionary(const fs::path &inDictFile)
 
 	try
 	{
-		mImpl = new CompoundFactoryImpl(inDictFile, mImpl);
+		mImpl = new CompoundFactoryImpl(inDictFile.string(), mImpl);
 	}
 	catch (const std::exception &ex)
 	{
@@ -1093,6 +1122,21 @@ void CompoundFactory::pushDictionary(const fs::path &inDictFile)
 	}
 }
 
+void CompoundFactory::pushDictionary(std::istream &is)
+{
+	mImpl = new CompoundFactoryImpl(is, mImpl);
+}
+
+void CompoundFactory::popDictionary()
+{
+	assert(mImpl != nullptr);
+	if (mImpl != nullptr)
+		mImpl = mImpl->pop();
+
+	// should not pop the last one!
+	assert(mImpl != nullptr);
+}
+
 // id is the three letter code
 const Compound *CompoundFactory::get(std::string id)
 {
