@@ -27,17 +27,17 @@
 #include <atomic>
 #include <mutex>
 
-#include "cif++/CifUtils.hpp"
+#include <cif++/utilities.hpp>
 
 #include "pdb-redo/ClipperWrapper.hpp"
 #include "pdb-redo/DistanceMap.hpp"
 #include "pdb-redo/Symmetry-2.hpp"
 
-//#define DEBUG_VOOR_BART
-
 namespace pdb_redo
 {
 
+using cif::point;
+
 // --------------------------------------------------------------------
 
 std::vector<clipper::RTop_orth> DistanceMap::AlternativeSites(const clipper::Spacegroup &spacegroup,
@@ -52,9 +52,9 @@ std::vector<clipper::RTop_orth> DistanceMap::AlternativeSites(const clipper::Spa
 	{
 		const auto &symop = spacegroup.symop(i);
 
-		for (int u : {-1, 0, 1})
-			for (int v : {-1, 0, 1})
-				for (int w : {-1, 0, 1})
+		for (int u : { -1, 0, 1 })
+			for (int v : { -1, 0, 1 })
+				for (int w : { -1, 0, 1 })
 				{
 					if (i == 0 and u == 0 and v == 0 and w == 0)
 						continue;
@@ -72,47 +72,78 @@ std::vector<clipper::RTop_orth> DistanceMap::AlternativeSites(const clipper::Spa
 
 // --------------------------------------------------------------------
 
-DistanceMap::DistanceMap(const Structure &p, const clipper::Spacegroup &spacegroup, const clipper::Cell &cell,
+std::tuple<point, float> calculateCenterAndRadius(const std::vector<std::tuple<size_t,point>> &atoms)
+{
+	std::vector<point> pts;
+	for (const auto &[ix, pt] : atoms)
+		pts.emplace_back(pt);
+
+	auto center = centroid(pts);
+	float radius = 0;
+
+	for (auto &pt : pts)
+	{
+		float d = static_cast<float>(distance(pt, center));
+		if (radius < d)
+			radius = d;
+	}
+
+	return std::make_tuple(center, radius);
+}
+
+// --------------------------------------------------------------------
+
+DistanceMap::DistanceMap(const cif::mm::structure &p, const clipper::Spacegroup &spacegroup, const clipper::Cell &cell,
 	float maxDistance)
-	: structure(p)
+	: mStructure(p)
 	, cell(cell)
 	, spacegroup(spacegroup)
 	, dim(0)
 	, mMaxDistance(maxDistance)
 	, mMaxDistanceSQ(maxDistance * maxDistance)
 {
-	auto &atoms = p.atoms();
-	dim = atoms.size();
+	using namespace cif::literals;
 
-	std::vector<clipper::Coord_orth> locations(dim);
+	// First collect the atoms from the datablock
+	std::vector<cif::row_handle> atoms;
+
+	auto &db = p.get_datablock();
+
+	for (auto a : p.atoms())
+		atoms.push_back(a.get_row());
+
+	dim = uint32_t(atoms.size());
+
+	std::vector<point> locations(dim);
 
 	// bounding box
-	Point pMin(std::numeric_limits<float>::max(), std::numeric_limits<float>::max(), std::numeric_limits<float>::max()),
+	point pMin(std::numeric_limits<float>::max(), std::numeric_limits<float>::max(), std::numeric_limits<float>::max()),
 		pMax(std::numeric_limits<float>::min(), std::numeric_limits<float>::min(), std::numeric_limits<float>::min());
 
 	for (auto &atom : atoms)
 	{
+		const auto &[id, x, y, z] = atom.get<std::string, float, float, float>("id", "Cartn_x", "Cartn_y", "Cartn_z");
+
 		size_t ix = index.size();
-		index[atom.id()] = ix;
-		rIndex[ix] = atom.id();
+		index[id] = ix;
+		rIndex[ix] = id;
 
-		locations[ix] = toClipper(atom.location());
+		cif::point pt{ x, y, z };
+		locations[ix] = pt;
 
-		auto pt = atom.location();
+		if (pMin.m_x > pt.m_x)
+			pMin.m_x = pt.m_x;
+		if (pMin.m_y > pt.m_y)
+			pMin.m_y = pt.m_y;
+		if (pMin.m_z > pt.m_z)
+			pMin.m_z = pt.m_z;
 
-		if (pMin.mX > pt.mX)
-			pMin.mX = pt.mX;
-		if (pMin.mY > pt.mY)
-			pMin.mY = pt.mY;
-		if (pMin.mZ > pt.mZ)
-			pMin.mZ = pt.mZ;
-
-		if (pMax.mX < pt.mX)
-			pMax.mX = pt.mX;
-		if (pMax.mY < pt.mY)
-			pMax.mY = pt.mY;
-		if (pMax.mZ < pt.mZ)
-			pMax.mZ = pt.mZ;
+		if (pMax.m_x < pt.m_x)
+			pMax.m_x = pt.m_x;
+		if (pMax.m_y < pt.m_y)
+			pMax.m_y = pt.m_y;
+		if (pMax.m_z < pt.m_z)
+			pMax.m_z = pt.m_z;
 	};
 
 	// correct locations so that the median of x, y and z are inside the cell
@@ -124,23 +155,23 @@ DistanceMap::DistanceMap(const Structure &p, const clipper::Spacegroup &spacegro
 		           : (c[dim / 2 - 1] + c[dim / 2]) / 2;
 	};
 
-	transform(locations.begin(), locations.end(), c.begin(), [](auto &l)
-		{ return static_cast<float>(l[0]); });
+	transform(locations.begin(), locations.end(), c.begin(), [](point &l)
+		{ return static_cast<float>(l.m_x); });
 	sort(c.begin(), c.end());
 	float mx = median();
 
-	transform(locations.begin(), locations.end(), c.begin(), [](auto &l)
-		{ return static_cast<float>(l[1]); });
+	transform(locations.begin(), locations.end(), c.begin(), [](point &l)
+		{ return static_cast<float>(l.m_y); });
 	sort(c.begin(), c.end());
 	float my = median();
 
-	transform(locations.begin(), locations.end(), c.begin(), [](auto &l)
-		{ return static_cast<float>(l[2]); });
+	transform(locations.begin(), locations.end(), c.begin(), [](point &l)
+		{ return static_cast<float>(l.m_z); });
 	sort(c.begin(), c.end());
 	float mz = median();
 
 	if (cif::VERBOSE > 1)
-		std::cerr << "median position of atoms: " << Point(mx, my, mz) << std::endl;
+		std::cerr << "median position of atoms: " << point(mx, my, mz) << std::endl;
 
 	auto calculateD = [&](float m, float c)
 	{
@@ -152,19 +183,19 @@ DistanceMap::DistanceMap(const Structure &p, const clipper::Spacegroup &spacegro
 		return d;
 	};
 
-	mD.mX = calculateD(mx, static_cast<float>(cell.a()));
-	mD.mY = calculateD(my, static_cast<float>(cell.b()));
-	mD.mZ = calculateD(mz, static_cast<float>(cell.c()));
+	mD.m_x = calculateD(mx, static_cast<float>(cell.a()));
+	mD.m_y = calculateD(my, static_cast<float>(cell.b()));
+	mD.m_z = calculateD(mz, static_cast<float>(cell.c()));
 
 	clipper::Coord_orth D = toClipper(mD);
 
-	if (mD.mX != 0 or mD.mY != 0 or mD.mZ != 0)
+	if (mD.m_x != 0 or mD.m_y != 0 or mD.m_z != 0)
 	{
-		if (cif::VERBOSE)
-			std::cerr << "moving coorinates by " << mD.mX << ", " << mD.mY << " and " << mD.mZ << std::endl;
+		if (cif::VERBOSE > 1)
+			std::cerr << "moving coorinates by " << mD.m_x << ", " << mD.m_y << " and " << mD.m_z << std::endl;
 
-		for_each(locations.begin(), locations.end(), [&](auto &p)
-			{ p += toClipper(mD); });
+		for_each(locations.begin(), locations.end(), [&](point &p)
+			{ p += mD; });
 	}
 
 	pMin -= mMaxDistance; // extend bounding box
@@ -174,61 +205,89 @@ DistanceMap::DistanceMap(const Structure &p, const clipper::Spacegroup &spacegro
 
 	DistMap dist;
 
-	std::vector<const Residue *> residues;
+	std::vector<std::tuple<cif::point, float, std::vector<std::tuple<size_t,point>>>> residues;
 
-	for (auto &poly : p.polymers())
+	// loop over poly_seq_scheme
+	for (const auto &[asymID, seqID] : db["pdbx_poly_seq_scheme"].rows<std::string, int>("asym_id", "seq_id"))
 	{
-		for (auto &m : poly)
+		std::vector<std::tuple<size_t,point>> rAtoms;
+		for (size_t i = 0; i < dim; ++i)
 		{
-			residues.emplace_back(&m);
+			if (atoms[i]["label_asym_id"] == asymID and atoms[i]["label_seq_id"] == seqID)
+				rAtoms.emplace_back(i, locations[i]);
+		}
+		
+		AddDistancesForAtoms(rAtoms, rAtoms, dist, 0);
 
-			// Add distances for atoms in this residue
-			AddDistancesForAtoms(m, m, dist, 0);
+		auto &&[center, radius] = calculateCenterAndRadius(rAtoms);
+		residues.emplace_back(center, radius, std::move(rAtoms));
+	}
+
+	// treat waters special
+	std::string water_entity_id = db["entity"].find1<std::string>("type"_key == "water", "id");
+
+	for (size_t i = 0; i < dim; ++i)
+	{
+		if (atoms[i]["label_entity_id"] == water_entity_id)
+		{
+			auto pt = locations[i];
+			residues.emplace_back(pt, 0.f, std::vector<std::tuple<size_t,point>>{ { i, pt } });
 		}
 	}
 
-	for (auto &r : p.getNonPolymers())
+	// loop over pdbx_nonpoly_scheme
+	for (const auto &[asymID, entityID] : db["pdbx_nonpoly_scheme"].rows<std::string, std::string>("asym_id", "entity_id"))
 	{
-		residues.emplace_back(&r);
+		if (entityID == water_entity_id)
+			continue;
 
-		// Add distances for atoms in this residue
-		AddDistancesForAtoms(r, r, dist, 0);
+		std::vector<std::tuple<size_t,point>> rAtoms;
+		for (size_t i = 0; i < dim; ++i)
+		{
+			if (atoms[i]["label_asym_id"] == asymID)
+				rAtoms.emplace_back(i, locations[i]);
+		}
+		
+		AddDistancesForAtoms(rAtoms, rAtoms, dist, 0);
+
+		auto &&[center, radius] = calculateCenterAndRadius(rAtoms);
+		residues.emplace_back(center, radius, std::move(rAtoms));
 	}
 
-	for (auto &r : p.getBranchResidues())
+	// loop over pdbx_branch_scheme
+	for (const auto &[asym_id, pdb_seq_num] : db["pdbx_branch_scheme"].rows<std::string, std::string>("asym_id", "pdb_seq_num"))
 	{
-		residues.emplace_back(&r);
+		std::vector<std::tuple<size_t,point>> rAtoms;
+		for (size_t i = 0; i < dim; ++i)
+		{
+			if (atoms[i]["label_asym_id"] == asym_id and atoms[i]["label_seq_id"] == pdb_seq_num)
+				rAtoms.emplace_back(i, locations[i]);
+		}
+		
+		AddDistancesForAtoms(rAtoms, rAtoms, dist, 0);
 
-		// Add distances for atoms in this residue
-		AddDistancesForAtoms(r, r, dist, 0);
+		auto &&[center, radius] = calculateCenterAndRadius(rAtoms);
+		residues.emplace_back(center, radius, std::move(rAtoms));
 	}
 
-	cif::Progress progress(residues.size() * residues.size(), "Creating distance std::map");
+	cif::Progress progress(residues.size() * (residues.size() - 1), "Creating distance map");
 
 	for (size_t i = 0; i + 1 < residues.size(); ++i)
 	{
-		auto &ri = *residues[i];
-
-		Point centerI;
-		float radiusI;
-		std::tie(centerI, radiusI) = ri.centerAndRadius();
+		const auto &[centerI, radiusI, atomsI] = residues[i];
 
 		for (size_t j = i + 1; j < residues.size(); ++j)
 		{
 			progress.consumed(1);
 
-			auto &rj = *residues[j];
+			const auto &[centerJ, radiusJ, atomsJ] = residues[j];
 
 			// first case, no symmetry operations
 
-			Point centerJ;
-			float radiusJ;
-			std::tie(centerJ, radiusJ) = rj.centerAndRadius();
-
-			auto d = Distance(centerI, centerJ) - radiusI - radiusJ;
+			auto d = distance(centerI, centerJ) - radiusI - radiusJ;
 			if (d < mMaxDistance)
 			{
-				AddDistancesForAtoms(ri, rj, dist, 0);
+				AddDistancesForAtoms(atomsI, atomsJ, dist, 0);
 				continue;
 			}
 
@@ -255,7 +314,7 @@ DistanceMap::DistanceMap(const Structure &p, const clipper::Spacegroup &spacegro
 			}
 
 			if (minR2 < mMaxDistance)
-				AddDistancesForAtoms(ri, rj, dist, kbest);
+				AddDistancesForAtoms(atomsI, atomsJ, dist, kbest);
 		}
 	}
 
@@ -269,10 +328,10 @@ DistanceMap::DistanceMap(const Structure &p, const clipper::Spacegroup &spacegro
 	size_t lastR = 0;
 	mIA.push_back(0);
 
-	for (auto &di : dist)
+	for (const auto &[key, value] : dist)
 	{
 		size_t col, row;
-		std::tie(row, col) = di.first;
+		std::tie(row, col) = key;
 
 		if (row != lastR) // new row
 		{
@@ -281,7 +340,7 @@ DistanceMap::DistanceMap(const Structure &p, const clipper::Spacegroup &spacegro
 			lastR = row;
 		}
 
-		mA.push_back(di.second);
+		mA.push_back(value);
 		mJA.push_back(col);
 	}
 
@@ -291,19 +350,18 @@ DistanceMap::DistanceMap(const Structure &p, const clipper::Spacegroup &spacegro
 
 // --------------------------------------------------------------------
 
-void DistanceMap::AddDistancesForAtoms(const Residue &a, const Residue &b, DistMap &dm, int32_t rtix)
+void DistanceMap::AddDistancesForAtoms(const std::vector<std::tuple<size_t,point>> &a, const std::vector<std::tuple<size_t,point>> &b, DistMap &dm, int32_t rtix)
 {
-	for (auto &aa : a.atoms())
+	for (const auto &[ixa, loc_a] : a)
 	{
-		clipper::Coord_orth pa = toClipper(aa.location());
-		size_t ixa = index[aa.id()];
+		clipper::Coord_orth pa = toClipper(loc_a);
 
-		for (auto &bb : b.atoms())
+		for (const auto &[ixb, loc_b] : b)
 		{
-			if (aa.id() == bb.id())
+			if (ixa == ixb)
 				continue;
 
-			clipper::Coord_orth pb = toClipper(bb.location());
+			clipper::Coord_orth pb = toClipper(loc_b);
 
 			if (rtix)
 				pb = pb.transform(mRtOrth[rtix]);
@@ -314,34 +372,32 @@ void DistanceMap::AddDistancesForAtoms(const Residue &a, const Residue &b, DistM
 
 			d = std::sqrt(d);
 
-			size_t ixb = index[bb.id()];
-
 			dm[std::make_tuple(ixa, ixb)] = std::make_tuple(d, rtix);
 			dm[std::make_tuple(ixb, ixa)] = std::make_tuple(d, -rtix);
 		}
 	}
 }
 
-float DistanceMap::operator()(const Atom &a, const Atom &b) const
+float DistanceMap::operator()(const std::string &a, const std::string &b) const
 {
 	size_t ixa, ixb;
 
 	try
 	{
-		ixa = index.at(a.id());
+		ixa = index.at(a);
 	}
 	catch (const std::out_of_range &ex)
 	{
-		throw std::runtime_error("atom " + a.id() + " not found in distance std::map");
+		throw std::runtime_error("atom " + a + " not found in distance map");
 	}
 
 	try
 	{
-		ixb = index.at(b.id());
+		ixb = index.at(b);
 	}
 	catch (const std::out_of_range &ex)
 	{
-		throw std::runtime_error("atom " + b.id() + " not found in distance std::map");
+		throw std::runtime_error("atom " + b + " not found in distance map");
 	}
 
 	//	if (ixb < ixa)
@@ -366,24 +422,30 @@ float DistanceMap::operator()(const Atom &a, const Atom &b) const
 	return 100.f;
 }
 
-std::vector<Atom> DistanceMap::near(const Atom &a, float maxDistance) const
+std::vector<cif::mm::atom> DistanceMap::near(const cif::mm::atom &atom, float maxDistance) const
 {
+	using namespace cif::literals;
+
 	assert(maxDistance <= mMaxDistance);
 	if (maxDistance > mMaxDistance)
 		throw std::runtime_error("Invalid max distance in DistanceMap::near");
 
+	std::string a_id = atom.id();
+	std::string alta = atom.get_label_alt_id();
+
 	size_t ixa;
 	try
 	{
-		ixa = index.at(a.id());
+		ixa = index.at(a_id);
 	}
 	catch (const std::out_of_range &ex)
 	{
-		throw std::runtime_error("atom " + a.id() + " not found in distance map");
+		throw std::runtime_error("atom " + a_id + " not found in distance map");
 	}
 
-	std::vector<Atom> result;
-	auto alta = a.labelAltID();
+	std::vector<cif::mm::atom> result;
+
+	auto &atom_site = atom.get_row().get_category();
 
 	for (size_t i = mIA[ixa]; i < mIA[ixa + 1]; ++i)
 	{
@@ -395,18 +457,21 @@ std::vector<Atom> DistanceMap::near(const Atom &a, float maxDistance) const
 			continue;
 
 		size_t ixb = mJA[i];
-		Atom b = structure.getAtomByID(rIndex.at(ixb));
 
-		auto altb = b.labelAltID();
+		std::string b_id = rIndex.at(ixb);
+		auto altb = atom_site.find1<std::string>("id"_key == b_id, "label_alt_id");
+
 		if (altb != alta and not altb.empty() and not alta.empty())
 			continue;
 
+		auto atom_b = mStructure.get_atom_by_id(b_id);
+
 		if (rti > 0)
-			result.emplace_back(symmetryCopy(b, mD, spacegroup, cell, mRtOrth.at(rti)));
+			result.emplace_back(symmetryCopy(atom_b, mD, spacegroup, cell, mRtOrth.at(rti)));
 		else if (rti < 0)
-			result.emplace_back(symmetryCopy(b, mD, spacegroup, cell, mRtOrth.at(-rti).inverse()));
+			result.emplace_back(symmetryCopy(atom_b, mD, spacegroup, cell, mRtOrth.at(-rti).inverse()));
 		else
-			result.emplace_back(b);
+			result.emplace_back(atom_b);
 	}
 
 	return result;
