@@ -28,28 +28,22 @@
 #include <fstream>
 #include <iomanip>
 
-#include <boost/algorithm/string.hpp>
-#include <boost/format.hpp>
-#include <boost/iostreams/copy.hpp>
-#include <boost/iostreams/filter/gzip.hpp>
-#include <boost/iostreams/filtering_stream.hpp>
-
 #include <clipper/clipper-ccp4.h>
 #include <clipper/clipper-contrib.h>
 
-#include "cif++/Cif++.hpp"
+#include <cif++.hpp>
 
 #include "pdb-redo/ClipperWrapper.hpp"
 #include "pdb-redo/MapMaker.hpp"
 #include "pdb-redo/ResolutionCalculator.hpp"
+#include "pdb-redo/Statistics.hpp"
+#include "pdb-redo/Symmetry-2.hpp"
 
 #ifdef _MSC_VER
 #include <io.h>
 #define mkstemp _mktemp
 #endif
 
-namespace io = boost::iostreams;
-namespace ba = boost::algorithm;
 namespace fs = std::filesystem;
 
 namespace pdb_redo
@@ -349,7 +343,7 @@ void Map<FTYPE>::read(const std::filesystem::path &f)
 	fs::path mapFile(f);
 	fs::path dataFile = mapFile;
 
-	if (cif::VERBOSE)
+	if (cif::VERBOSE > 0)
 		std::cout << "Reading map from " << mapFile << std::endl;
 
 	if (mapFile.extension() == ".gz")
@@ -359,13 +353,7 @@ void Map<FTYPE>::read(const std::filesystem::path &f)
 		fs::path p = mapFile.parent_path();
 		std::string s = mapFile.filename().string();
 
-		io::filtering_stream<io::input> in;
-
-		std::ifstream fi(mapFile);
-
-		in.push(io::gzip_decompressor());
-
-		in.push(fi);
+		cif::gzio::ifstream in(mapFile);
 
 		char tmpFileName[] = "/tmp/map-tmp-XXXXXX";
 		if (mkstemp(tmpFileName) < 0)
@@ -373,7 +361,11 @@ void Map<FTYPE>::read(const std::filesystem::path &f)
 
 		dataFile = fs::path(tmpFileName);
 		std::ofstream out(dataFile);
-		io::copy(in, out);
+
+		if (not in.is_open() or not out.is_open())
+			throw std::runtime_error("Could not handle compressed map file");
+
+		out << in.rdbuf();
 	}
 
 	if (not fs::exists(dataFile))
@@ -414,6 +406,65 @@ void Map<FTYPE>::write_masked(const std::filesystem::path &f, clipper::Grid_rang
 	write_masked(file, r);
 }
 
+// --------------------------------------------------------------------
+
+template <typename FTYPE>
+Map<FTYPE> Map<FTYPE>::masked(const cif::mm::structure &structure, const std::vector<cif::mm::atom> &atoms) const
+{
+	Map<FTYPE> result(*this);
+
+	auto rtops = AlternativeSites(getSpacegroup(structure.get_datablock()), getCell(structure.get_datablock()));
+
+	for (auto &atom : atoms)
+	{
+		float radius = cif::atom_type_traits(atom.get_type()).radius(cif::radius_type::van_der_waals);
+		if (std::isnan(radius))
+			radius = cif::atom_type_traits(atom.get_type()).radius(cif::radius_type::calculated);
+		if (std::isnan(radius))	// TODO: now what?
+			radius = 200;
+
+		auto cloc = toClipper(atom.get_location());
+
+		for (auto &rt : rtops)
+		{
+			auto rcloc = cloc.transform(rt);
+
+			iterateGrid(toClipper(toPoint(rcloc)), radius, result.mMap,
+				[&result, radiusSq = radius * radius, a = atom.get_location()](auto iw)
+				{
+					cif::point p = toPoint(iw.coord_orth());
+
+					if (distance_squared(a, p) < radiusSq)
+						result.mMap[iw] = -10;
+				});
+		}
+	}
+
+	return result;
+}
+
+template <typename FTYPE>
+float Map<FTYPE>::z_weighted_density(const cif::mm::structure &structure, const std::vector<cif::mm::atom> &atoms) const
+{
+	FTYPE result = 0;
+
+	for (auto &atom : atoms)
+	{
+		auto co = toClipper(atom.get_location());
+		auto a_cf = co.coord_frac(mMap.cell());
+		auto a_cm = a_cf.coord_map(mMap.grid_sampling());
+
+		FTYPE dv;
+		clipper::Interp_nearest::interp(mMap, a_cm, dv);
+
+		result += dv * static_cast<int>(atom.get_type() == cif::atom_type::D ? cif::atom_type::H : atom.get_type());
+	}
+
+	return result;
+}
+
+// --------------------------------------------------------------------
+
 template class Map<float>;
 template class Map<double>;
 
@@ -437,13 +488,13 @@ void MapMaker<FTYPE>::loadMTZ(const fs::path &f, float samplingRate,
 {
 	fs::path hklin(f);
 
-	if (cif::VERBOSE)
+	if (cif::VERBOSE > 0)
 		std::cerr << "Reading map from " << hklin << std::endl
-				  << "  with labels: FB: " << ba::join(fbLabels, ",") << std::endl
-				  << "  with labels: FD: " << ba::join(fdLabels, ",") << std::endl
-				  << "  with labels: FA: " << ba::join(faLabels, ",") << std::endl
-				  << "  with labels: FO: " << ba::join(foLabels, ",") << std::endl
-				  << "  with labels: FC: " << ba::join(fcLabels, ",") << std::endl;
+				  << "  with labels: FB: " << cif::join(fbLabels, ",") << std::endl
+				  << "  with labels: FD: " << cif::join(fdLabels, ",") << std::endl
+				  << "  with labels: FA: " << cif::join(faLabels, ",") << std::endl
+				  << "  with labels: FO: " << cif::join(foLabels, ",") << std::endl
+				  << "  with labels: FC: " << cif::join(fcLabels, ",") << std::endl;
 
 	fs::path dataFile = hklin;
 
@@ -454,13 +505,7 @@ void MapMaker<FTYPE>::loadMTZ(const fs::path &f, float samplingRate,
 		fs::path p = hklin.parent_path();
 		std::string s = hklin.filename().string();
 
-		io::filtering_stream<io::input> in;
-
-		std::ifstream fi(hklin);
-
-		in.push(io::gzip_decompressor());
-
-		in.push(fi);
+		cif::gzio::ifstream in(hklin);
 
 		char tmpFileName[] = "/tmp/mtz-tmp-XXXXXX";
 		if (mkstemp(tmpFileName) < 0)
@@ -468,13 +513,14 @@ void MapMaker<FTYPE>::loadMTZ(const fs::path &f, float samplingRate,
 
 		dataFile = fs::path(tmpFileName);
 		std::ofstream out(dataFile);
-		io::copy(in, out);
+		
+		out << in.rdbuf();
 	}
 
 	if (not fs::exists(dataFile))
 		throw std::runtime_error("Could not open mtz file " + hklin.string());
 
-	const std::string kBasePath("/%1%/%2%/[%3%]");
+	const std::string kBasePath("/%s/%s/[%s]");
 
 	using clipper::CCP4MTZfile;
 
@@ -506,23 +552,23 @@ void MapMaker<FTYPE>::loadMTZ(const fs::path &f, float samplingRate,
 	}
 
 	mtzin.import_hkl_data(mFbData,
-		(boost::format(kBasePath) % "*" % "*" % ba::join(fbLabels, ",")).str());
+		cif::format(kBasePath, "*", "*", cif::join(fbLabels, ",")).str());
 	mtzin.import_hkl_data(mFdData,
-		(boost::format(kBasePath) % "*" % "*" % ba::join(fdLabels, ",")).str());
+		cif::format(kBasePath, "*", "*", cif::join(fdLabels, ",")).str());
 	if (hasFAN)
 		mtzin.import_hkl_data(mFaData,
-			(boost::format(kBasePath) % "*" % "*" % ba::join(faLabels, ",")).str());
+			cif::format(kBasePath, "*", "*", cif::join(faLabels, ",")).str());
 	mtzin.import_hkl_data(mFoData,
-		(boost::format(kBasePath) % "*" % "*" % ba::join(foLabels, ",")).str());
+		cif::format(kBasePath, "*", "*", cif::join(foLabels, ",")).str());
 	mtzin.import_hkl_data(mFcData,
-		(boost::format(kBasePath) % "*" % "*" % ba::join(fcLabels, ",")).str());
+		cif::format(kBasePath, "*", "*", cif::join(fcLabels, ",")).str());
 
 	if (hasFREE)
 		mtzin.import_hkl_data(mFreeData,
-			(boost::format(kBasePath) % "*" % "*" % "FREE").str());
+			cif::format(kBasePath, "*", "*", "FREE").str());
 
 	mtzin.import_hkl_data(mPhiFomData,
-		(boost::format(kBasePath) % "*" % "*" % "PHWT,FOM").str());
+		cif::format(kBasePath, "*", "*", "PHWT,FOM").str());
 
 	mtzin.close_read();
 
@@ -547,6 +593,9 @@ void MapMaker<FTYPE>::loadMTZ(const fs::path &f, float samplingRate,
 			mResLow = res;
 	}
 
+	if (mResLow == 0 and mResHigh == 99)
+		throw std::runtime_error("Empty Fo map");
+
 	//	fixMTZ();
 
 	mGrid.init(spacegroup, cell,
@@ -568,7 +617,7 @@ void MapMaker<FTYPE>::loadMTZ(const fs::path &f, float samplingRate,
 		faMap.fft_from(mFaData);
 	}
 
-	if (cif::VERBOSE)
+	if (cif::VERBOSE > 0)
 	{
 		std::cerr << "Read Xmaps with sampling rate: " << samplingRate << std::endl
 				  << "  stored resolution: " << mHKLInfo.resolution().limit() << std::endl
@@ -597,6 +646,11 @@ void MapMaker<FTYPE>::loadMaps(const fs::path &fbMapFile, const fs::path &fdMapF
 
 	if (not mFb.cell().equals(mFd.cell()))
 		throw std::runtime_error("Fb and Fd map do not contain the same cell");
+
+	clipper::Resolution reso(reshi);
+
+	mHKLInfo.init(mFb.spacegroup(), mFb.cell(), reso, true);
+	mGrid = mFb.get().grid_sampling();
 }
 
 // --------------------------------------------------------------------
@@ -614,7 +668,7 @@ std::ostream &operator<<(std::ostream &os, const clipper::HKL &hkl)
 
 template <typename FTYPE>
 void MapMaker<FTYPE>::calculate(const fs::path &hklin,
-	const Structure &structure, bool noBulk, AnisoScalingFlag anisoScaling,
+	const cif::mm::structure &structure, bool noBulk, AnisoScalingFlag anisoScaling,
 	float samplingRate, bool electronScattering,
 	std::initializer_list<std::string> foLabels, std::initializer_list<std::string> freeLabels)
 {
@@ -633,8 +687,8 @@ void MapMaker<FTYPE>::loadFoFreeFromReflectionsFile(const fs::path &hklin)
 {
 	using clipper::HKL;
 
-	cif::File reflnsFile(hklin);
-	auto &reflns = reflnsFile.firstDatablock();
+	cif::file reflnsFile(hklin);
+	auto &reflns = reflnsFile.front();
 
 	//	m_xname = reflns["exptl_crystal"].front()["id"].as<std::string>();
 	//	m_pname = reflns["entry"].front()["id"].as<std::string>();
@@ -708,7 +762,7 @@ void MapMaker<FTYPE>::loadFoFreeFromReflectionsFile(const fs::path &hklin)
 
 		if (ix < 0)
 		{
-			if (cif::VERBOSE)
+			if (cif::VERBOSE > 0)
 				std::cerr << "Ignoring hkl(" << h << ", " << k << ", " << l << ")" << std::endl;
 			continue;
 		}
@@ -753,10 +807,10 @@ template <typename FTYPE>
 void MapMaker<FTYPE>::loadFoFreeFromMTZFile(const fs::path &hklin,
 	std::initializer_list<std::string> foLabels, std::initializer_list<std::string> freeLabels)
 {
-	if (cif::VERBOSE)
+	if (cif::VERBOSE > 0)
 		std::cerr << "Recalculating maps from " << hklin << std::endl;
 
-	const std::string kBasePath("/%1%/%2%/[%3%]");
+	const std::string kBasePath("/%s/%s/[%s]");
 
 	using clipper::CCP4MTZfile;
 
@@ -765,9 +819,9 @@ void MapMaker<FTYPE>::loadFoFreeFromMTZFile(const fs::path &hklin,
 
 	mtzin.import_hkl_info(mHKLInfo);
 	mtzin.import_hkl_data(mFoData,
-		(boost::format(kBasePath) % "*" % "*" % ba::join(foLabels, ",")).str());
+		cif::format(kBasePath, "*", "*", cif::join(foLabels, ",")).str());
 	mtzin.import_hkl_data(mFreeData,
-		(boost::format(kBasePath) % "*" % "*" % ba::join(freeLabels, ",")).str());
+		cif::format(kBasePath, "*", "*", cif::join(freeLabels, ",")).str());
 
 	mtzin.close_read();
 }
@@ -775,7 +829,7 @@ void MapMaker<FTYPE>::loadFoFreeFromMTZFile(const fs::path &hklin,
 // --------------------------------------------------------------------
 
 template <typename FTYPE>
-void MapMaker<FTYPE>::recalc(const Structure &structure,
+void MapMaker<FTYPE>::recalc(const cif::mm::structure &structure,
 	bool noBulk, AnisoScalingFlag anisoScaling,
 	float samplingRate, bool electronScattering)
 {
@@ -792,7 +846,7 @@ void MapMaker<FTYPE>::recalc(const Structure &structure,
 
 	if (not electronScattering)
 	{
-		auto &exptl = structure.getFile().data()["exptl"];
+		auto &exptl = structure.get_category("exptl");
 		electronScattering = not exptl.empty() and exptl.front()["method"] == "ELECTRON CRYSTALLOGRAPHY";
 	}
 
@@ -809,7 +863,7 @@ void MapMaker<FTYPE>::recalc(const Structure &structure,
 		clipper::SFcalc_obs_bulk<float> sfcb;
 		sfcb(mFcData, mFoData, atoms);
 
-		if (cif::VERBOSE)
+		if (cif::VERBOSE > 0)
 			std::cerr << "Bulk correction volume: " << sfcb.bulk_frac() << std::endl
 					  << "Bulk correction factor: " << sfcb.bulk_scale() << std::endl;
 	}
@@ -823,7 +877,7 @@ void MapMaker<FTYPE>::recalc(const Structure &structure,
 		else
 			sfscl(mFcData, mFoData); // scale Fcal
 
-		if (cif::VERBOSE)
+		if (cif::VERBOSE > 0)
 			std::cerr << "Anisotropic scaling:" << std::endl
 					  << sfscl.u_aniso_orth(F).format() << std::endl;
 	}
@@ -871,7 +925,7 @@ void MapMaker<FTYPE>::recalc(const Structure &structure,
 	if (cif::VERBOSE > 1)
 		std::cerr << "calculated reshi = " << mResHigh << " reslo = " << mResLow << std::endl;
 
-	samplingRate /= 2;
+	// samplingRate /= 2;
 
 	mGrid.init(spacegroup, cell,
 		mHKLInfo.resolution(), samplingRate); // define grid
@@ -885,11 +939,12 @@ void MapMaker<FTYPE>::recalc(const Structure &structure,
 	fdMap.init(spacegroup, cell, mGrid); // define map
 	fdMap.fft_from(mFdData);             // generate map
 
-	if (cif::VERBOSE)
+	if (cif::VERBOSE > 0)
 	{
 		std::cerr << "Read Xmaps with sampling rate: " << samplingRate << std::endl
-				  << "  resolution: " << mResHigh
-				  << std::endl;
+				  << "  resolution: " << mResHigh << std::endl
+				  << "  cell: " << cell.format() << std::endl
+				  << "  grid: " << mGrid.format() << std::endl;
 
 		printStats();
 	}
@@ -923,7 +978,7 @@ void MapMaker<FTYPE>::fixMTZ()
 
 	// first run the tests to see if we need to fix anything
 
-	if (cif::VERBOSE)
+	if (cif::VERBOSE > 0)
 		std::cerr << "Testing MTZ file" << std::endl;
 
 	for (auto ih = mFbData.first(); not ih.last(); ih.next())
@@ -953,14 +1008,14 @@ void MapMaker<FTYPE>::fixMTZ()
 			if (tests[T10] and std::abs(FM - FC) > 0.05)
 			{
 				tests[T10] = false;
-				if (cif::VERBOSE)
+				if (cif::VERBOSE > 0)
 					std::cerr << "Test 10 failed at " << ih.hkl() << std::endl;
 			}
 
 			if (tests[T11] and std::abs(FD) > 0.05)
 			{
 				tests[T11] = false;
-				if (cif::VERBOSE)
+				if (cif::VERBOSE > 0)
 					std::cerr << "Test 11 failed at " << ih.hkl() << std::endl;
 			}
 		}
@@ -969,35 +1024,35 @@ void MapMaker<FTYPE>::fixMTZ()
 			if (tests[C5] and std::abs(FC + FM - 2 * WFO) > 0.05)
 			{
 				tests[C5] = false;
-				if (cif::VERBOSE)
+				if (cif::VERBOSE > 0)
 					std::cerr << "Test C5 failed at " << ih.hkl() << std::endl;
 			}
 
 			if (tests[C6] and std::abs(FM - WFO) > 0.05)
 			{
 				tests[C6] = false;
-				if (cif::VERBOSE)
+				if (cif::VERBOSE > 0)
 					std::cerr << "Test C6 failed at " << ih.hkl() << std::endl;
 			}
 
 			if (tests[C7] and std::abs(FC + FD - WFO) > 0.05)
 			{
 				tests[C7] = false;
-				if (cif::VERBOSE)
+				if (cif::VERBOSE > 0)
 					std::cerr << "Test C7 failed at " << ih.hkl() << std::endl;
 			}
 
 			if (tests[C8] and std::abs(FC + 0.5 * FD - WFO) > 0.05)
 			{
 				tests[C8] = false;
-				if (cif::VERBOSE)
+				if (cif::VERBOSE > 0)
 					std::cerr << "Test C8 failed at " << ih.hkl() << std::endl;
 			}
 
-			if (tests[C9] and (1.01 * FC + Gd - WFO) < -0.05)
+			if (tests[C9] and (1.01 * FC + FD - WFO) < -0.05)
 			{
 				tests[C9] = false;
-				if (cif::VERBOSE)
+				if (cif::VERBOSE > 0)
 					std::cerr << "Test C9 failed at " << ih.hkl() << std::endl;
 			}
 		}
@@ -1006,28 +1061,28 @@ void MapMaker<FTYPE>::fixMTZ()
 			if (tests[A1] and std::abs(FC + FM - 2 * WFO) > 0.05)
 			{
 				tests[A1] = false;
-				if (cif::VERBOSE)
+				if (cif::VERBOSE > 0)
 					std::cerr << "Test A1 failed at " << ih.hkl() << std::endl;
 			}
 
 			if (tests[A2] and 1.01 * FC + FM - 2 * WFO < -0.05)
 			{
 				tests[A2] = false;
-				if (cif::VERBOSE)
+				if (cif::VERBOSE > 0)
 					std::cerr << "Test A2 failed at " << ih.hkl() << std::endl;
 			}
 
 			if (tests[A3] and std::abs(FM - FD - WFO) > 0.05)
 			{
 				tests[A3] = false;
-				if (cif::VERBOSE)
+				if (cif::VERBOSE > 0)
 					std::cerr << "Test A3 failed at " << ih.hkl() << std::endl;
 			}
 
 			if (tests[A4] and std::abs(FM - 0.5 * FD - WFO) > 0.05)
 			{
 				tests[A4] = false;
-				if (cif::VERBOSE)
+				if (cif::VERBOSE > 0)
 					std::cerr << "Test A4 failed at " << ih.hkl() << std::endl;
 			}
 		}
