@@ -32,17 +32,9 @@
 #include <filesystem>
 #include <fstream>
 
-#include <zeep/xml/document.hpp>
-#include <zeep/xml/serialize.hpp>
-
-#include <zeep/json/element.hpp>
-#include <zeep/json/parser.hpp>
-#include <zeep/json/serializer.hpp>
-
 #include "pdb-redo/SkipList.hpp"
 
 namespace fs = std::filesystem;
-namespace zx = zeep::xml;
 
 namespace pdb_redo
 {
@@ -56,20 +48,12 @@ void writeOLDSkipList(std::ostream &os, const SkipList &list)
 		os << res.auth_asym_id << res.auth_seq_id << (res.pdbx_PDB_ins_code and *res.pdbx_PDB_ins_code ? *res.pdbx_PDB_ins_code : ' ') << ':';
 }
 
-void writeJSONSkipList(std::ostream &os, const SkipList &list)
-{
-	zeep::json::element e;
-	zeep::json::to_element(e, list);
-	os << e;
-}
-
 void writeCIFSkipList(std::ostream &os, const SkipList &list)
 {
-	cif::File file;
-	auto db = new cif::Datablock("skip");
-	file.append(db);
+	cif::file file;
+	auto &db = file.emplace_back("skip");
 
-	auto &&[cat, ignore] = db->emplace("skip_list");
+	auto &&[cat, ignore] = db.emplace("skip_list");
 
 	for (auto &res : list)
 		cat->emplace({{"auth_asym_id", res.auth_asym_id},
@@ -93,10 +77,6 @@ void writeSkipList(std::ostream &os, const SkipList &list, SkipListFormat format
 			writeOLDSkipList(os, list);
 			break;
 
-		case SkipListFormat::JSON:
-			writeJSONSkipList(os, list);
-			break;
-
 		case SkipListFormat::CIF:
 			writeCIFSkipList(os, list);
 			break;
@@ -158,26 +138,14 @@ SkipList readOLDSkipList(std::istream &is)
 	return result;
 }
 
-SkipList readJSONSkipList(std::istream &is)
-{
-	SkipList result;
-
-	zeep::json::element e;
-	zeep::json::parse_json(is, e);
-
-	from_element(e, result);
-
-	return result;
-}
-
 SkipList readCIFSkipList(std::istream &is)
 {
 	SkipList result;
 
-	cif::File file;
+	cif::file file;
 	file.load(is);
 
-	auto &db = file.firstDatablock();
+	auto &db = file["skip"];
 	auto &cat = db["skip_list"];
 
 	for (const auto &[auth_asym_id, auth_comp_id, auth_seq_id, pdbx_PDB_ins_code, label_asym_id, label_comp_id, label_seq_id] :
@@ -197,18 +165,7 @@ SkipList readSkipList(std::istream &is)
 	}
 	catch (const std::exception &e)
 	{
-		if (cif::VERBOSE)
-			std::cerr << e.what() << std::endl;
-		is.rdbuf()->pubseekpos(0);
-	}
-
-	try
-	{
-		return readJSONSkipList(is);
-	}
-	catch (const std::exception &e)
-	{
-		if (cif::VERBOSE)
+		if (cif::VERBOSE > 0)
 			std::cerr << e.what() << std::endl;
 		is.rdbuf()->pubseekpos(0);
 	}
