@@ -27,9 +27,10 @@
 #include <fstream>
 #include <numeric>
 
-#include "cif++/Structure.hpp"
+#include <cif++.hpp>
 
 #include "pdb-redo/AtomShape.hpp"
+#include "pdb-redo/BondMap.hpp"
 #include "pdb-redo/ClipperWrapper.hpp"
 #include "pdb-redo/Statistics.hpp"
 
@@ -42,8 +43,7 @@ using clipper::Coord_grid;
 using clipper::Coord_orth;
 using clipper::Xmap;
 
-using mmcif::AtomTypeTraits;
-using mmcif::BondMap;
+using cif::atom_type_traits;
 
 // --------------------------------------------------------------------
 
@@ -185,7 +185,7 @@ double errsol(double a)
 class PointWeightFunction
 {
   public:
-	PointWeightFunction(Point center, float atomRadius)
+	PointWeightFunction(cif::point center, float atomRadius)
 		: m_Center(center)
 		, m_Radius(atomRadius)
 	{
@@ -194,9 +194,9 @@ class PointWeightFunction
 		m_P[2] = P{-0.9507f, 2, 0, 2};
 	}
 
-	float operator()(Point p) const
+	float operator()(cif::point p) const
 	{
-		float d = Distance(m_Center, p);
+		float d = distance(m_Center, p);
 		d /= m_Radius;
 
 		float result = 0;
@@ -224,38 +224,13 @@ class PointWeightFunction
 		float m, c, b, x;
 	};
 
-	Point m_Center;
+	cif::point m_Center;
 	float m_Radius;
 	P m_P[3];
 };
 
 // --------------------------------------------------------------------
 
-template <typename F>
-void iterateGrid(const Coord_orth &p, float r, const Xmap<float> &m, F &&func)
-{
-	using namespace clipper;
-
-	Coord_frac fp = p.coord_frac(m.cell());
-
-	Coord_frac o = Coord_orth(r, r, r).coord_frac(m.cell());
-	o[0] = std::abs(o[0]);
-	o[1] = std::abs(o[1]);
-	o[2] = std::abs(o[2]);
-
-	Coord_frac fMin = fp - o, fMax = fp + o;
-	Coord_map mMin = fMin.coord_map(m.grid_sampling()), mMax = fMax.coord_map(m.grid_sampling());
-	Coord_grid gMin = mMin.floor(), gMax = mMax.ceil();
-
-	auto i0 = Xmap_base::Map_reference_coord(m, gMin);
-	for (auto iu = i0; iu.coord().u() <= gMax[0]; iu.next_u())
-		for (auto iv = iu; iv.coord().v() <= gMax[1]; iv.next_v())
-			for (auto iw = iv; iw.coord().w() <= gMax[2]; iw.next_w())
-				func(iw);
-}
-
-// --------------------------------------------------------------------
-
 struct AtomGridData
 {
 	AtomGridData(const Coord_grid &gp, double density)
@@ -295,17 +270,6 @@ struct AtomDataSums
 		return *this;
 	}
 
-	friend AtomDataSums operator*(const AtomDataSums &a, float factor)
-	{
-		return {
-			static_cast<size_t>(std::round(a.ngrid * factor)),
-			{a.rfSums[0] * factor, a.rfSums[1] * factor},
-			{a.edSums[0] * factor, a.edSums[1] * factor},
-			{a.ccSums[0] * factor, a.ccSums[1] * factor, a.ccSums[2] * factor},
-			{a.rgSums[0] * factor, a.rgSums[1] * factor},
-			{a.swSums[0] * factor, a.swSums[1] * factor, a.swSums[2] * factor}};
-	}
-
 	double cc() const
 	{
 		double s = (ccSums[1] - (edSums[0] * edSums[0]) / ngrid) * (ccSums[2] - (edSums[1] * edSums[1]) / ngrid);
@@ -314,25 +278,26 @@ struct AtomDataSums
 
 	double srg() const
 	{
-		double rg = std::sqrt(rgSums[0] / rgSums[1]);
+		double rgsq = rgSums[0] / rgSums[1];
+		double rg = std::sqrt(rgsq);
 
-		return std::sqrt(swSums[0] - rg * rg * swSums[1] + 0.5 * rg * rg * rg * rg * swSums[2]) / (rg * rgSums[1]);
+		return std::sqrt(swSums[0] - rgsq * swSums[1] + 0.5 * rgsq * rgsq * swSums[2]) / (rg * rgSums[1]);
 	}
 };
 
 struct AtomData
 {
-	AtomData(Atom atom, float radius)
+	AtomData(cif::mm::atom atom, float radius)
 		: atom(atom)
-		, asymID(atom.labelAsymID())
-		, seqID(atom.labelSeqID())
-		, authSeqID(atom.authSeqID())
+		, asymID(atom.get_label_asym_id())
+		, seqID(atom.get_label_seq_id())
+		, authSeqID(atom.get_auth_seq_id())
 		, radius(radius)
-		, occupancy(atom.occupancy())
+		, occupancy(atom.get_occupancy())
 	{
 	}
 
-	Atom atom;
+	cif::mm::atom atom;
 	std::string asymID;
 	int seqID;
 	std::string authSeqID; // required for waters
@@ -374,65 +339,28 @@ std::tuple<float, float> CalculateMapStatistics(const Xmap<float> &f)
 class BoundingBox
 {
   public:
-	//	BoundingBox(const Structure& structure, const std::vector<std::tuple<std::string,int,std::string,std::string>>& residues, float margin)
-	//	{
-	//		mXMin = mYMin = mZMin = std::numeric_limits<float>::max();
-	//		mXMax = mYMax = mZMax = std::numeric_limits<float>::min();
-	//
-	//		for (auto& r: residues)
-	//		{
-	//			int seqID;
-	//			std::string asymID, compID, pdbID;
-	//			std::tie(asymID, seqID, compID, pdbID) = r;
-	//
-	//			Residue res(structure, compID, asymID, seqID);
-	//			for (auto& atom: res.atoms())
-	//			{
-	//				auto l = atom.location();
-	//				if (mXMin > l.mX)
-	//					mXMin = l.mX;
-	//				if (mXMax < l.mX)
-	//					mXMax = l.mX;
-	//				if (mYMin > l.mY)
-	//					mYMin = l.mY;
-	//				if (mYMax < l.mY)
-	//					mYMax = l.mY;
-	//				if (mZMin > l.mZ)
-	//					mZMin = l.mZ;
-	//				if (mZMax < l.mZ)
-	//					mZMax = l.mZ;
-	//			}
-	//		}
-	//
-	//		mXMin -= margin;
-	//		mXMax += margin;
-	//		mYMin -= margin;
-	//		mYMax += margin;
-	//		mZMin -= margin;
-	//		mZMax += margin;
-	//	}
 
 	template <class List>
-	BoundingBox(const Structure &structure, List atoms, float margin)
+	BoundingBox(const cif::mm::structure &structure, List atoms, float margin)
 	{
 		mXMin = mYMin = mZMin = std::numeric_limits<float>::max();
 		mXMax = mYMax = mZMax = std::numeric_limits<float>::min();
 
 		for (auto &atom : atoms)
 		{
-			auto l = atom.location();
-			if (mXMin > l.mX)
-				mXMin = l.mX;
-			if (mXMax < l.mX)
-				mXMax = l.mX;
-			if (mYMin > l.mY)
-				mYMin = l.mY;
-			if (mYMax < l.mY)
-				mYMax = l.mY;
-			if (mZMin > l.mZ)
-				mZMin = l.mZ;
-			if (mZMax < l.mZ)
-				mZMax = l.mZ;
+			auto l = atom.get_location();
+			if (mXMin > l.m_x)
+				mXMin = l.m_x;
+			if (mXMax < l.m_x)
+				mXMax = l.m_x;
+			if (mYMin > l.m_y)
+				mYMin = l.m_y;
+			if (mYMax < l.m_y)
+				mYMax = l.m_y;
+			if (mZMin > l.m_z)
+				mZMin = l.m_z;
+			if (mZMax < l.m_z)
+				mZMax = l.m_z;
 		}
 
 		mXMin -= margin;
@@ -443,9 +371,9 @@ class BoundingBox
 		mZMax += margin;
 	}
 
-	bool contains(const Point &p) const
+	bool contains(const cif::point &p) const
 	{
-		return p.mX >= mXMin and p.mX <= mXMax and p.mY >= mYMin and p.mY <= mYMax and p.mZ >= mZMin and p.mZ <= mZMax;
+		return p.m_x >= mXMin and p.m_x <= mXMax and p.m_y >= mYMin and p.m_y <= mYMax and p.m_z >= mZMin and p.m_z <= mZMax;
 	}
 
   private:
@@ -453,7 +381,7 @@ class BoundingBox
 };
 // --------------------------------------------------------------------
 
-StatsCollector::StatsCollector(const MapMaker<float> &mm, Structure &structure, bool electronScattering)
+StatsCollector::StatsCollector(const MapMaker<float> &mm, cif::mm::structure &structure, bool electronScattering)
 	: mStructure(structure)
 	, mMapMaker(mm)
 	, mElectronScattering(electronScattering)
@@ -469,6 +397,10 @@ StatsCollector::StatsCollector(const MapMaker<float> &mm, Structure &structure,
 
 void StatsCollector::initialize()
 {
+	// easiest way to prime this map:
+	for (auto &asym_id : mStructure.get_datablock()["struct_asym"].rows<std::string>("id"))
+		mRmsScaled[asym_id] = { 1, 1 };
+
 	mMeanDensityFb = mMapMaker.fb().meanDensity();
 	mRMSDensityFb = mMapMaker.fb().rmsDensity();
 	mRMSDensityFd = mMapMaker.fd().rmsDensity();
@@ -493,11 +425,11 @@ void StatsCollector::initialize()
 
 	for (auto &a : mStructure.atoms())
 	{
-		auto t = a.type();
-		if (t <= mmcif::He)
+		auto t = a.get_type();
+		if (t <= cif::atom_type::He)
 			continue;
 
-		float w = a.occupancy() * t;
+		float w = a.get_occupancy() * t;
 
 		if (w <= 0)
 			continue;
@@ -529,7 +461,7 @@ void StatsCollector::initialize()
 
 		if (cif::VERBOSE > 2)
 			std::cerr << (atomData.size() + 1) << '\t'
-					  << AtomTypeTraits(atom.type()).symbol() << '\t'
+					  << atom_type_traits(atom.get_type()).symbol() << '\t'
 					  << radius << std::endl;
 
 		atomData.emplace_back(atom, radius);
@@ -616,16 +548,15 @@ void StatsCollector::initialize()
 
 std::vector<ResidueStatistics> StatsCollector::collect() const
 {
-	std::vector<std::tuple<std::string, int, std::string, std::string>> residues;
-	std::vector<Atom> atoms;
+	std::vector<std::tuple<std::string, int, std::string>> residues;
+	std::vector<cif::mm::atom> atoms;
 
 	for (auto atom : mStructure.atoms())
 	{
-		if (atom.isWater())
+		if (atom.is_water())
 			continue;
 
-		auto k = std::make_tuple(atom.labelAsymID(), atom.labelSeqID(), atom.labelCompID(), atom.authSeqID());
-		//		auto k = std::make_tuple(atom.authAsymID(), atom.property<std::string>("auth_seq_id"), atom.authCompID());
+		auto k = std::make_tuple(atom.get_label_asym_id(), atom.get_label_seq_id(), atom.get_auth_seq_id());
 
 		if (residues.empty() or residues.back() != k)
 		{
@@ -640,43 +571,73 @@ std::vector<ResidueStatistics> StatsCollector::collect() const
 
 std::vector<ResidueStatistics> StatsCollector::collect(const std::string &asymID, int resFirst, int resLast, bool authNameSpace) const
 {
-	std::vector<std::tuple<std::string, int, std::string, std::string>> residues;
-	std::vector<Atom> atoms;
+	residue_list residues;
+	std::vector<cif::mm::atom> atoms;
+
+	// for (auto atom : mStructure.atoms())
+	// {
+	// 	if (atom.is_water())
+	// 		continue;
+
+	// 	if (authNameSpace)
+	// 	{
+	// 		int auth_seq_id = stoi(atom.get_auth_seq_id());
+
+	// 		if (atom.get_auth_asym_id() != asymID or auth_seq_id < resFirst or auth_seq_id > resLast)
+	// 			continue;
+	// 	}
+	// 	else
+	// 	{
+	// 		if (atom.get_label_asym_id() != asymID or atom.get_label_seq_id() < resFirst or atom.get_label_seq_id() > resLast)
+	// 			continue;
+	// 	}
+
+	// 	auto k = std::make_tuple(atom.get_label_asym_id(), atom.get_label_seq_id(), atom.get_auth_seq_id());
+
+	// 	if (residues.empty() or residues.back() != k)
+	// 	{
+	// 		residues.emplace_back(move(k));
+	// 		atoms.emplace_back(std::move(atom));
+	// 	}
+	// }
 
 	for (auto atom : mStructure.atoms())
 	{
-		if (atom.isWater())
+		if (atom.is_water())
 			continue;
 
 		if (authNameSpace)
 		{
-			int authSeqID = stoi(atom.authSeqID());
+			int auth_seq_id = stoi(atom.get_auth_seq_id());
 
-			if (atom.authAsymID() != asymID or authSeqID < resFirst or authSeqID > resLast)
+			if (atom.get_auth_asym_id() != asymID or auth_seq_id < resFirst or auth_seq_id > resLast)
 				continue;
 		}
 		else
 		{
-			if (atom.labelAsymID() != asymID or atom.labelSeqID() < resFirst or atom.labelSeqID() > resLast)
+			if (atom.get_label_asym_id() != asymID or atom.get_label_seq_id() < resFirst or atom.get_label_seq_id() > resLast)
 				continue;
 		}
 
-		auto k = std::make_tuple(atom.labelAsymID(), atom.labelSeqID(), atom.labelCompID(), atom.authSeqID());
-		//		auto k = std::make_tuple(atom.authAsymID(), atom.property<std::string>("auth_seq_id"), atom.authCompID());
+		auto k = std::make_tuple(atom.get_label_asym_id(), atom.get_label_seq_id(), atom.get_auth_seq_id());
 
 		if (residues.empty() or residues.back() != k)
-		{
 			residues.emplace_back(move(k));
-			atoms.emplace_back(std::move(atom));
-		}
+	}
+
+	for (const auto &[asymID, seqID, authSeqID] : residues)
+	{
+		auto &res = mStructure.get_residue(asymID, seqID, authSeqID);
+
+		for (auto atom : res.unique_atoms())
+			atoms.push_back(atom);
 	}
 
 	BoundingBox bbox(mStructure, atoms, 5.0f);
 	return collect(residues, bbox, false);
 }
 
-std::vector<ResidueStatistics> StatsCollector::collect(const std::vector<std::tuple<std::string, int, std::string, std::string>> &residues,
-	BoundingBox &bbox, bool addWaters) const
+std::vector<ResidueStatistics> StatsCollector::collect(const residue_list &residues, BoundingBox &bbox, bool addWaters) const
 {
 	std::vector<AtomData> atomData;
 
@@ -684,12 +645,12 @@ std::vector<ResidueStatistics> StatsCollector::collect(const std::vector<std::tu
 
 	for (auto atom : mStructure.atoms())
 	{
-		if (atom.isWater())
+		if (atom.is_water())
 		{
 			if (not addWaters)
 				continue;
 		}
-		else if (not bbox.contains(atom.location()))
+		else if (not bbox.contains(atom.get_location()))
 			continue;
 
 		AtomShape shape(atom, mResHigh, mResLow, mElectronScattering);
@@ -698,7 +659,7 @@ std::vector<ResidueStatistics> StatsCollector::collect(const std::vector<std::tu
 
 		if (cif::VERBOSE > 2)
 			std::cerr << (atomData.size() + 1) << '\t'
-					  << AtomTypeTraits(atom.type()).symbol() << '\t'
+					  << atom_type_traits(atom.get_type()).symbol() << '\t'
 					  << radius << std::endl;
 
 		atomData.emplace_back(atom, radius);
@@ -710,8 +671,12 @@ std::vector<ResidueStatistics> StatsCollector::collect(const std::vector<std::tu
 	std::vector<ResidueStatistics> result;
 
 	// And now collect the per residue information
-	for (const auto &[asymID, seqID, compID, authSeqID] : residues)
+	for (const auto &[asymID, seqID, authSeqID] : residues)
 	{
+		// TODO: Need to do something with hetero residues (alternate compound types)
+		auto &res = mStructure.get_residue(asymID, seqID, authSeqID);
+		auto compID = res.get_compound_id();
+
 		AtomDataSums sums;
 
 		std::vector<const AtomData *> resAtomData;
@@ -736,30 +701,25 @@ std::vector<ResidueStatistics> StatsCollector::collect(const std::vector<std::tu
 
 					for (auto d : resAtomData)
 					{
-						if (d->atom.labelAtomID() != compAtom)
+						if (d->atom.get_label_atom_id() != compAtom)
 							continue;
 
-						float o_d = d->occupancy;
-
-						if (o_d == 0)
-							continue;
-
-						if (o_d == 1)
-							sums += d->sums;
-						else
-							sums += d->sums * o_d;
+						// We used to factor in the occupancy here, but that seems to be incorrect
+						// since occupancy was already used.
+						// sums += d->sums * d->occupancy;
 
+						sums += d->sums;
 						break;
 					}
 
 					resAtomData.erase(
 						std::remove_if(resAtomData.begin(), resAtomData.end(), [id = compAtom](const AtomData *d)
-							{ return d->atom.labelAtomID() == id; }),
+							{ return d->atom.get_label_atom_id() == id; }),
 						resAtomData.end());
 				}
 			}
 		}
-		catch (const mmcif::BondMapException &ex)
+		catch (const BondMapException &ex)
 		{
 			std::cerr << "Missing information for compound '" << compID << '\'' << std::endl
 					  << ex.what() << std::endl;
@@ -769,17 +729,14 @@ std::vector<ResidueStatistics> StatsCollector::collect(const std::vector<std::tu
 
 		// atoms that were present but not part of the Compound
 		for (auto d : resAtomData)
-		{
-			if (d->occupancy == 1)
-				sums += d->sums;
-			else
-				sums += d->sums * d->occupancy;
-		}
+			sums += d->sums;
 
 		// EDIA
 
-		auto &res = mStructure.getResidue(asymID, compID, seqID);
-		auto alts = res.getAlternateIDs();
+		std::set<std::string> alts;
+
+		if (not missing.count(compID))
+			alts = res.get_alternate_ids();
 
 		if (alts.empty())
 			alts.insert("");
@@ -802,7 +759,7 @@ std::vector<ResidueStatistics> StatsCollector::collect(const std::vector<std::tu
 					continue;
 				}
 
-				auto altd = d.atom.labelAltID();
+				auto altd = d.atom.get_label_alt_id();
 				if (altd.empty() or altd == alt)
 					resAtomData.push_back(&d);
 			}
@@ -827,7 +784,7 @@ std::vector<ResidueStatistics> StatsCollector::collect(const std::vector<std::tu
 
 					for (auto d : resAtomData)
 					{
-						if (d->atom.labelAtomID() != compAtom)
+						if (d->atom.get_label_atom_id() != compAtom)
 							continue;
 
 						occSum += d->occupancy;
@@ -873,13 +830,12 @@ std::vector<ResidueStatistics> StatsCollector::collect(const std::vector<std::tu
 	{
 		for (const auto &d : atomData)
 		{
-			const Atom &atom = d.atom;
+			const cif::mm::atom &atom = d.atom;
 
-			if (not atom.isWater())
+			if (not atom.is_water())
 				continue;
 
-			result.emplace_back(ResidueStatistics{d.asymID, d.seqID, "HOH",
-				atom.authSeqID(),
+			result.emplace_back(ResidueStatistics{d.asymID, d.seqID, "HOH", d.authSeqID,
 				(d.sums.rfSums[0] / d.sums.rfSums[1]),         // rsr
 				d.sums.srg(),                                  // srsr
 				d.sums.cc(),                                   // rsccs
@@ -892,9 +848,9 @@ std::vector<ResidueStatistics> StatsCollector::collect(const std::vector<std::tu
 	return result;
 }
 
-ResidueStatistics StatsCollector::collect(std::initializer_list<const Residue *> residues) const
+ResidueStatistics StatsCollector::collect(std::initializer_list<const cif::mm::residue *> residues) const
 {
-	std::vector<Atom> atoms;
+	std::vector<cif::mm::atom> atoms;
 	for (auto &r : residues)
 		for (auto a : r->atoms())
 			atoms.push_back(a);
@@ -902,13 +858,13 @@ ResidueStatistics StatsCollector::collect(std::initializer_list<const Residue *>
 	return collect(atoms);
 }
 
-ResidueStatistics StatsCollector::collect(std::initializer_list<Atom> atoms) const
+ResidueStatistics StatsCollector::collect(std::initializer_list<cif::mm::atom> atoms) const
 {
-	std::vector<Atom> v(atoms);
+	std::vector<cif::mm::atom> v(atoms);
 	return collect(v);
 }
 
-ResidueStatistics StatsCollector::collect(const std::vector<Atom> &atoms) const
+ResidueStatistics StatsCollector::collect(const std::vector<cif::mm::atom> &atoms) const
 {
 	std::vector<AtomData> atomData;
 
@@ -916,7 +872,7 @@ ResidueStatistics StatsCollector::collect(const std::vector<Atom> &atoms) const
 
 	for (auto atom : mStructure.atoms())
 	{
-		if (not bb.contains(atom.location()))
+		if (not bb.contains(atom.get_location()))
 			continue;
 
 		AtomShape shape(atom, mResHigh, mResLow, mElectronScattering);
@@ -925,7 +881,7 @@ ResidueStatistics StatsCollector::collect(const std::vector<Atom> &atoms) const
 
 		if (cif::VERBOSE > 2)
 			std::cerr << (atomData.size() + 1) << '\t'
-					  << AtomTypeTraits(atom.type()).symbol() << '\t'
+					  << atom_type_traits(atom.get_type()).symbol() << '\t'
 					  << radius << std::endl;
 
 		atomData.emplace_back(atom, radius);
@@ -943,7 +899,7 @@ ResidueStatistics StatsCollector::collect(const std::vector<Atom> &atoms) const
 
 		auto ci = find_if(atomData.begin(), atomData.end(),
 			[=](auto &d)
-			{ return d.asymID == atom.labelAsymID() and d.seqID == atom.labelSeqID() and d.atom.labelAtomID() == atom.labelAtomID(); });
+			{ return d.asymID == atom.get_label_asym_id() and d.seqID == atom.get_label_seq_id() and d.atom.get_label_atom_id() == atom.get_label_atom_id(); });
 
 		if (ci == atomData.end())
 			continue;
@@ -983,27 +939,31 @@ void StatsCollector::sumDensity(std::vector<AtomData> &atomData,
 	{
 		auto &atom = data.atom;
 
+		if (atom.get_occupancy() == 0)
+			continue;
+		
 		AtomShape shape(atom, mResHigh, mResLow, mElectronScattering);
 
 		std::string asymID = data.asymID;
-		if (atom.isWater())
+		if (atom.is_water())
 			asymID = "0";
-
+		
 		auto radius = data.radius;
 		double sumDensity = 0;
 
-		iterateGrid(toClipper(atom.location()), radius, Fb, [&](Xmap_base::Map_reference_coord &iw)
+		iterateGrid(toClipper(atom.get_location()), radius, Fb, [&](Xmap_base::Map_reference_coord &iw)
 			{
 			auto p = toPoint(iw.coord_orth());
 			
-			double d = Distance(p, atom.location());
+			double d = distance(p, atom.get_location());
 
 			if (d <= radius)
 			{
 				double density = shape.calculatedDensity(p);
 				
-				assert(not std::isnan(density));
-	
+				if (std::isnan(density))
+					return;
+
 				gridPointDensity[iw.coord()] += density;
 				data.points.emplace_back(iw.coord(), density);
 				
@@ -1034,7 +994,11 @@ void StatsCollector::collectSums(std::vector<AtomData> &atomData, GridPtDataMap
 		{
 			++d.sums.ngrid;
 
-			double e = gp.density / gridPointDensity[gp.p];
+			auto gpd = gridPointDensity[gp.p];
+			if (gpd == 0)
+				continue;
+
+			double e = gp.density / gpd;
 			double t = e * mSZ / rmsScaledF.second;
 
 			Xmap_base::Map_reference_coord ix(Fb, gp.p);
@@ -1089,7 +1053,7 @@ void StatsCollector::calculate(std::vector<AtomData> &atomData) const
 // --------------------------------------------------------------------
 
 EDIAStatsCollector::EDIAStatsCollector(MapMaker<float> &mm,
-	Structure &structure, bool electronScattering, const BondMap &bondMap)
+	cif::mm::structure &structure, bool electronScattering, const BondMap &bondMap)
 	: StatsCollector(mm, structure, electronScattering)
 	, mDistanceMap(structure, mm.spacegroup(), mm.cell(), 3.5f)
 	, mBondMap(bondMap)
@@ -1124,19 +1088,19 @@ EDIAStatsCollector::EDIAStatsCollector(MapMaker<float> &mm,
 	else
 		ediaBFactor = kAverageBFactors[i];
 
-	if (cif::VERBOSE)
+	if (cif::VERBOSE > 1)
 		std::cerr << "Calculating radii with B Factor " << ediaBFactor << std::endl;
 
 	for (auto atom : mStructure.atoms())
 	{
-		if (mRadii.count(atom.type()))
+		if (mRadii.count(atom.get_type()))
 			continue;
 
 		AtomShape shape(atom, mResHigh, mResLow, mElectronScattering, ediaBFactor);
-		mRadii[atom.type()] = shape.radius();
+		mRadii[atom.get_type()] = shape.radius();
 
-		if (cif::VERBOSE)
-			std::cerr << "Radius for atom with type " << AtomTypeTraits(atom.type()).symbol() << " is " << mRadii[atom.type()] << std::endl;
+		if (cif::VERBOSE > 1)
+			std::cerr << "Radius for atom with type " << atom_type_traits(atom.get_type()).symbol() << " is " << mRadii[atom.get_type()] << std::endl;
 	}
 }
 
@@ -1149,10 +1113,10 @@ void EDIAStatsCollector::calculate(std::vector<AtomData> &atomData) const
 
 	struct lessAtom
 	{
-		bool operator()(const Atom &a, const Atom &b) const { return a.id().compare(b.id()) < 0; }
+		bool operator()(const cif::mm::atom &a, const cif::mm::atom &b) const { return a.id().compare(b.id()) < 0; }
 	};
 
-	typedef std::set<Atom, lessAtom> atomSet;
+	typedef std::set<cif::mm::atom, lessAtom> atomSet;
 
 	// Calculate EDIA scores
 
@@ -1161,26 +1125,26 @@ void EDIAStatsCollector::calculate(std::vector<AtomData> &atomData) const
 	for (auto &data : atomData)
 	{
 		auto &atom = data.atom;
-		float radius = mRadii.at(atom.type());
+		float radius = mRadii.at(atom.get_type());
 
 		//		if (cif::VERBOSE > 2)
 		//			std::cerr << (atomData.size() + 1) << '\t'
-		//				 << AtomTypeTraits(atom.type()).symbol() << '\t'
+		//				 << atom_type_traits(atom.get_type()).symbol() << '\t'
 		//				 << radius << std::endl;
-		//
-		PointWeightFunction w(atom.location(), radius);
 
-		std::vector<Atom> atomsNearBy = mDistanceMap.near(atom, 3.5f);
+		PointWeightFunction w(atom.get_location(), radius);
+
+		std::vector<cif::mm::atom> atomsNearBy = mDistanceMap.near(atom, 3.5f);
 
 		std::vector<PointWeightFunction> wn;
 		for (auto a : atomsNearBy)
-			wn.emplace_back(a.location(), mRadii.at(a.type()));
+			wn.emplace_back(a.get_location(), mRadii.at(a.get_type()));
 
 		float ediaSum[2] = {};
 
-		iterateGrid(toClipper(atom.location()), radius, Fb, [&](auto iw)
-			{
-			Point p = toPoint(iw.coord_orth());
+		iterateGrid(toClipper(atom.get_location()), radius, Fb, [&](auto iw)
+		{
+			cif::point p = toPoint(iw.coord_orth());
 			
 			// EDIA calculations
 			auto fb = Fb[iw];
@@ -1234,12 +1198,12 @@ void EDIAStatsCollector::calculate(std::vector<AtomData> &atomData) const
 				else
 				{
 					float sumpb = accumulate(I.begin(), I.end(), 0.f,
-						[p](float s, const Atom& b) -> float
+						[p](float s, const cif::mm::atom& b) -> float
 						{
-							return s + Distance(p, b.location());
+							return s + distance(p, b.get_location());
 						});
 
-					o = 1 - Distance(atom.location(), p) / sumpb;
+					o = 1 - distance(atom.get_location(), p) / sumpb;
 				}
 			}
 			else if (D.count(atom) and S.empty())
@@ -1249,21 +1213,19 @@ void EDIAStatsCollector::calculate(std::vector<AtomData> &atomData) const
 				else
 				{
 					float sumpb = accumulate(D.begin(), D.end(), 0.f,
-						[p](float s, const Atom& b) -> float
+						[p](float s, const cif::mm::atom& b) -> float
 						{
-							return s + Distance(p, b.location());
+							return s + distance(p, b.get_location());
 						});
 
-					o = 1 - Distance(atom.location(), p) / sumpb;
+					o = 1 - distance(atom.get_location(), p) / sumpb;
 				}
 			}
 
-//					if (cif::VERBOSE > 2)
-//						std::cout << Point(p) << ":\td: " << xmap[iw] << "\tz: " << z << "\to: " << o << "\tzraw: " << ((xmap[iw] - meanDensity) / rmsDensity) << "\twp: " << wp << std::endl;
-			
 			ediaSum[0] += z * wp * o;
 			if (wp > 0)
-				ediaSum[1] += wp; });
+				ediaSum[1] += wp;
+		});
 
 		data.edia = ediaSum[0] / ediaSum[1];
 		if (data.edia < 0)
